@@ -33,6 +33,13 @@
   */
 #include <mail_stream.h>
 
+ /*
+  * Postfix TLS library.
+  */
+#ifdef USE_TLS
+#include <tls.h>
+#endif
+
  /*
   * Variables that keep track of conversation state. There is only one SMTP
   * conversation at a time, so the state variables can be made global. And
@@ -50,6 +57,7 @@ typedef struct {
     char   *name;			/* name for access control */
     char   *addr;			/* address for access control */
     char   *namaddr;			/* name[address] */
+    char   *rfc_addr;			/* address for RFC 2821 */
     char   *protocol;			/* email protocol */
     char   *helo_name;			/* helo/ehlo parameter */
     char   *ident;			/* message identifier */
@@ -65,6 +73,7 @@ typedef struct SMTPD_STATE {
     char   *name;			/* client hostname */
     char   *addr;			/* client host address string */
     char   *namaddr;			/* combined name and address */
+    char   *rfc_addr;			/* address for RFC 2821 */
     int     peer_code;			/* 2=ok, 4=soft, 5=hard */
     int     error_count;		/* reset after DOT */
     int     error_mask;			/* client errors */
@@ -140,6 +149,18 @@ typedef struct SMTPD_STATE {
      * XFORWARD server state.
      */
     SMTPD_XFORWARD_ATTR xforward;	/* up-stream logging info */
+
+    /*
+     * TLS related state.
+     */
+#ifdef USE_TLS
+    int     tls_use_tls;		/* can use TLS */
+    int     tls_enforce_tls;		/* must use TLS */
+    int     tls_auth_only;		/* use SASL over TLS only */
+    TLScontext_t *tls_context;		/* TLS session state */
+    tls_info_t tls_info;		/* legacy */
+#endif
+
 } SMTPD_STATE;
 
 #define SMTPD_STATE_XFORWARD_INIT  (1<<0)	/* xforward preset done */
@@ -234,7 +255,7 @@ extern void smtpd_peer_reset(SMTPD_STATE *state);
 	(((s)->xforward.flags & SMTPD_STATE_XFORWARD_CLIENT_MASK) ? \
 	    (s)->xforward.a : (s)->a)
 
-#define FORWARD_ADDR(s)		FORWARD_CLIENT_ATTR((s), addr)
+#define FORWARD_ADDR(s)		FORWARD_CLIENT_ATTR((s), rfc_addr)
 #define FORWARD_NAME(s)		FORWARD_CLIENT_ATTR((s), name)
 #define FORWARD_NAMADDR(s)	FORWARD_CLIENT_ATTR((s), namaddr)
 #define FORWARD_PROTO(s)	FORWARD_CLIENT_ATTR((s), protocol)
@@ -267,4 +288,11 @@ extern int smtpd_input_transp_mask;
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	TLS support originally by:
+/*	Lutz Jaenicke
+/*	BTU Cottbus
+/*	Allgemeine Elektrotechnik
+/*	Universitaetsplatz 3-4
+/*	D-03044 Cottbus, Germany
 /*--*/
