@@ -82,6 +82,7 @@
   * so that it can be accessed by call-back routines.
   */
 int     TLScontext_index = -1;
+int     TLSpeername_index = -1;
 
 /* tls_print_errors - print and clear the error stack */
 
@@ -126,7 +127,7 @@ void    tls_info_callback(const SSL *s, int where, int ret)
 	str = "unknown";
 
     if (where & SSL_CB_LOOP) {
-	msg_info("%s:%s", str, SSL_state_string_long((SSL *) s));
+	msg_info("%s:%s", str, SSL_state_string_long(s));
     } else if (where & SSL_CB_ALERT) {
 	str = (where & SSL_CB_READ) ? "read" : "write";
 	if ((ret & 0xff) != SSL3_AD_CLOSE_NOTIFY)
@@ -136,10 +137,10 @@ void    tls_info_callback(const SSL *s, int where, int ret)
     } else if (where & SSL_CB_EXIT) {
 	if (ret == 0)
 	    msg_info("%s:failed in %s",
-		     str, SSL_state_string_long((SSL *) s));
+		     str, SSL_state_string_long(s));
 	else if (ret < 0) {
 	    msg_info("%s:error in %s",
-		     str, SSL_state_string_long((SSL *) s));
+		     str, SSL_state_string_long(s));
 	}
     }
 }
@@ -227,11 +228,4 @@ long    tls_bio_dump_cb(BIO *bio, int cmd, const char *argp, int argi,
     return (ret);
 }
 
-#else
-
- /*
-  * Broken linker workaround.
-  */
-int     tls_dummy_for_broken_linkers;
-
 #endif
