@@ -200,6 +200,11 @@ struct ipv4addrinfo {
     struct sockaddr_in sin;
 };
 
+ /*
+  * Make nulls more descriptive.
+  */
+#define NO_SERVICE	((char *) 0)
+
  /*
   * When we're not interested in service ports, we must pick a socket type
   * otherwise getaddrinfo() will give us duplicate results: one set for TCP,
@@ -680,7 +685,6 @@ void    myaddrinfo_control(int name,...)
 	    msg_panic("%s: bad name %d", myname, name);
 	}
     }
-    va_end(ap);
 }
 
 #ifdef EMULATE_IPV4_ADDRINFO
